@@ -1,31 +1,40 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
+"""ffpuppet helper utilities"""
 
 from json import load as json_load
 from logging import getLogger
-from os import access, chmod, environ, mkdir, remove, W_OK
+from os import W_OK, access, chmod, environ, mkdir, remove, scandir
 from os.path import abspath, basename, expanduser, isdir, isfile
-from os.path import join as pathjoin, normcase, realpath
+from os.path import join as pathjoin
+from os.path import normcase, realpath
 from platform import system
 from re import compile as re_compile
 from shutil import copyfile, copytree, rmtree
 from stat import S_IWUSR
 from tempfile import mkdtemp
 from time import sleep, time
-
 from xml.etree import ElementTree
-from psutil import AccessDenied, NoSuchProcess, Process, process_iter
 
+from psutil import AccessDenied, NoSuchProcess, Process, process_iter
 
 LOG = getLogger(__name__)
 
 __author__ = "Tyson Smith"
-__all__ = ("check_prefs", "create_profile", "get_processes", "onerror",
-           "prepare_environment", "wait_on_files")
+__all__ = (
+    "check_prefs",
+    "create_profile",
+    "files_in_use",
+    "get_processes",
+    "onerror",
+    "prepare_environment",
+    "wait_on_files",
+    "warn_open",
+)
 
 
-class SanitizerConfig(object):
+class SanitizerConfig:  # pylint: disable=missing-docstring
     re_delim = re_compile(r":(?![\\|/])")
 
     __slots__ = ("_options",)
@@ -45,8 +54,21 @@ class SanitizerConfig(object):
             overwrite (bool): Overwrite existing value.
 
         Returns:
-            str: Colon separated list of options.
+            None
         """
+        assert flag and isinstance(flag, str)
+        assert isinstance(value, str)
+        if ":" in value or " " in value:
+            assert self.is_quoted(value), "%s (%s) must be quoted" % (value, flag)
+        elif value and (
+            value[0] == "'" or value[0] == '"' or value[-1] == "'" or value[-1] == '"'
+        ):
+            assert self.is_quoted(value), "unbalanced quotes on %s (%s)" % (value, flag)
+        # sanity check paths
+        if flag in ("external_symbolizer_path", "suppressions"):
+            path = abspath(expanduser(value.strip("'\"")))
+            if not isfile(path):
+                raise IOError("%r (%s) does not exist" % (path, flag))
         if flag not in self._options or overwrite:
             self._options[flag] = value
 
@@ -62,7 +84,7 @@ class SanitizerConfig(object):
         """
         if token.startswith("'") and token.endswith("'"):
             return True
-        if token.startswith("\"") and token.endswith("\""):
+        if token.startswith('"') and token.endswith('"'):
             return True
         return False
 
@@ -75,22 +97,13 @@ class SanitizerConfig(object):
         Returns:
             None
         """
+        self._options.clear()
         if not options:
             return
-        assert " " not in options, "*SAN_OPTIONS should not contain spaces"
         for option in self.re_delim.split(options):
             try:
-                opt_name, opt_value = option.split("=")
-                if ":" in opt_value:
-                    assert self.is_quoted(opt_value), "%s value must be quoted" % opt_name
-                # add a sanity check for suppression files
-                if opt_name == "suppressions":
-                    sup_file = abspath(expanduser(opt_value.strip("'\"")))
-                    if not isfile(sup_file):
-                        raise IOError("Suppressions file %r does not exist" % sup_file)
-                    opt_value = "'%s'" % sup_file
-                self._options[opt_name] = opt_value
-            except ValueError:
+                self.add(*option.split("=", maxsplit=1))
+            except TypeError:
                 LOG.warning("Malformed option %r", option)
 
     @property
@@ -149,17 +162,17 @@ def check_prefs(prof_prefs, input_prefs):
 
 
 def configure_sanitizers(env, target_dir, log_path):
-    """Update *SAN_OPTIONS entires in env.
+    """Update *SAN_OPTIONS entries in env.
 
     Args:
-        target_dir (str): Location to find llvm-symbolizer.
+        target_dir (str): Directory containing llvm-symbolizer.
         log_path (str): Location to write sanitizer logs to.
 
     Returns:
         None
     """
     # https://github.com/google/sanitizers/wiki/SanitizerCommonFlags
-    common_flags = (
+    common_flags = [
         ("abort_on_error", "false"),
         ("allocator_may_return_null", "true"),
         ("disable_coredump", "true"),
@@ -168,7 +181,28 @@ def configure_sanitizers(env, target_dir, log_path):
         ("handle_sigbus", "true"),  # set to be safe
         ("handle_sigfpe", "true"),  # set to be safe
         ("handle_sigill", "true"),  # set to be safe
-        ("symbolize", "true"))
+        ("max_allocation_size_mb", "512"),
+        # requires background thread so only works on Linux for now...
+        # https://github.com/llvm/llvm-project/blob/main/compiler-rt/lib/
+        # sanitizer_common/sanitizer_common_libcdep.cpp#L116
+        ("soft_rss_limit_mb", "5000"),
+        ("symbolize", "true"),
+    ]
+    if system().startswith("Windows"):
+        symbolizer_bin = "llvm-symbolizer.exe"
+    else:
+        symbolizer_bin = "llvm-symbolizer"
+    # use packaged llvm-symbolizer if available
+    if isfile(pathjoin(target_dir, symbolizer_bin)):
+        common_flags.append(
+            (
+                "external_symbolizer_path",
+                "'%s'" % (pathjoin(target_dir, symbolizer_bin),),
+            )
+        )
+    else:
+        # assume system llvm-symbolizer will be used
+        LOG.debug("llvm-symbolizer not found in %r", target_dir)
 
     # setup Address Sanitizer options if not set manually
     # https://github.com/google/sanitizers/wiki/AddressSanitizerFlags
@@ -176,21 +210,26 @@ def configure_sanitizers(env, target_dir, log_path):
     asan_config.load_options(env.get("ASAN_OPTIONS"))
     for flag in common_flags:
         asan_config.add(*flag)
-    #asan_config.add("alloc_dealloc_mismatch", "false")  # different defaults per OS
+    # different defaults per OS
+    # asan_config.add("alloc_dealloc_mismatch", "false")
     asan_config.add("check_initialization_order", "true")
-    #asan_config.add("detect_stack_use_after_return", "true")  # https://bugzil.la/1057551
-    #asan_config.add("detect_stack_use_after_scope", "true")
+    # https://bugzil.la/1057551
+    # asan_config.add("detect_stack_use_after_return", "true")
+    # asan_config.add("detect_stack_use_after_scope", "true")
     asan_config.add("detect_invalid_pointer_pairs", "1")
     asan_config.add("detect_leaks", "false")
     # log_path is required for FFPuppet logging to function properly
     if "log_path" in asan_config:
-        LOG.warning("ASAN_OPTIONS=log_path is used internally and cannot be set externally")
+        LOG.warning(
+            "ASAN_OPTIONS=log_path is used internally and cannot be set externally"
+        )
     asan_config.add("log_path", "'%s'" % log_path, overwrite=True)
     # attempt to save some memory during deep stack allocations
     asan_config.add("malloc_context_size", "20")
     asan_config.add("sleep_before_dying", "0")
     asan_config.add("strict_init_order", "true")
-    asan_config.add("strict_string_checks", "true")  # breaks old builds (esr52)
+    # breaks old builds (esr52)
+    asan_config.add("strict_string_checks", "true")
     env["ASAN_OPTIONS"] = asan_config.options
 
     # setup Leak Sanitizer options if not set manually
@@ -206,7 +245,9 @@ def configure_sanitizers(env, target_dir, log_path):
     tsan_config.load_options(env.get("TSAN_OPTIONS"))
     tsan_config.add("halt_on_error", "1")
     if "log_path" in tsan_config:
-        LOG.warning("TSAN_OPTIONS=log_path is used internally and cannot be set externally")
+        LOG.warning(
+            "TSAN_OPTIONS=log_path is used internally and cannot be set externally"
+        )
     tsan_config.add("log_path", "'%s'" % log_path, overwrite=True)
     env["TSAN_OPTIONS"] = tsan_config.options
 
@@ -216,35 +257,27 @@ def configure_sanitizers(env, target_dir, log_path):
     for flag in common_flags:
         ubsan_config.add(*flag)
     if "log_path" in ubsan_config:
-        LOG.warning("UBSAN_OPTIONS=log_path is used internally and cannot be set externally")
+        LOG.warning(
+            "UBSAN_OPTIONS=log_path is used internally and cannot be set externally"
+        )
     ubsan_config.add("log_path", "'%s'" % log_path, overwrite=True)
     ubsan_config.add("print_stacktrace", "1")
     env["UBSAN_OPTIONS"] = ubsan_config.options
 
-    if "ASAN_SYMBOLIZER_PATH" not in env:
-        # ASAN_SYMBOLIZER_PATH only needs to be set on platforms other than Windows
-        if not system().lower().startswith("windows"):
-            symbolizer_bin = pathjoin(target_dir, "llvm-symbolizer")
-            if isfile(symbolizer_bin):
-                env["ASAN_SYMBOLIZER_PATH"] = symbolizer_bin
-        elif not pathjoin(target_dir, "llvm-symbolizer.exe"):
-            LOG.warning("llvm-symbolizer.exe should be next to the target binary")
-    elif "ASAN_SYMBOLIZER_PATH" in env and not isfile(env["ASAN_SYMBOLIZER_PATH"]):
-        LOG.warning("Invalid ASAN_SYMBOLIZER_PATH (%s)", env["ASAN_SYMBOLIZER_PATH"])
 
-
-def create_profile(extension=None, prefs_js=None, template=None):
+def create_profile(extension=None, prefs_js=None, template=None, working_path=None):
     """Create a profile to be used with Firefox.
 
     Args:
         extension (str): Path to an extension to be installed.
         prefs_js (str): Path to the prefs.js file to install in the profile.
         template (str): Path to an existing profile directory to use.
+        working_path (str): Used as base directory for temporary files.
 
     Returns:
         str: Path to directory to be used as a profile.
     """
-    profile = mkdtemp(prefix="ffprof_")
+    profile = mkdtemp(dir=working_path, prefix="ffprofile_")
     try:
         if template is not None:
             LOG.debug("using profile template: %r", template)
@@ -291,8 +324,10 @@ def create_profile(extension=None, prefs_js=None, template=None):
                         LOG.debug("Failed to parse manifest.json: %s", exc)
                 elif isfile(pathjoin(ext, "install.rdf")):
                     try:
-                        xmlns = {"x": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
-                                 "em": "http://www.mozilla.org/2004/em-rdf#"}
+                        xmlns = {
+                            "x": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
+                            "em": "http://www.mozilla.org/2004/em-rdf#",
+                        }
                         tree = ElementTree.parse(pathjoin(ext, "install.rdf"))
                         assert tree.getroot().tag == "{%s}RDF" % xmlns["x"]
                         ids = tree.findall("./x:Description/em:id", namespaces=xmlns)
@@ -301,19 +336,46 @@ def create_profile(extension=None, prefs_js=None, template=None):
                     except (AssertionError, IOError, ElementTree.ParseError) as exc:
                         LOG.debug("Failed to parse install.rdf: %s", exc)
                 if ext_name is None:
-                    raise RuntimeError("Failed to find extension id in manifest: %r" % ext)
+                    raise RuntimeError(
+                        "Failed to find extension id in manifest: %r" % ext
+                    )
                 copytree(abspath(ext), pathjoin(profile, "extensions", ext_name))
             else:
                 raise RuntimeError("Unknown extension: %r" % ext)
-    except:
+    except Exception:
         # cleanup on failure
         rmtree(profile, True)
         raise
     return profile
 
 
+def files_in_use(check_files, path_fix, procs):
+    """Check if any of the given files are open by any of the given processes.
+
+    Args:
+        check_files (iterable(str)): Files path to check.
+        path_fix (callable): Function to format paths.
+        procs (iterable(Process)): Processes to scan for open files.
+
+    Yields:
+        tuple: Path of file, pid of process and process name.
+    """
+    if check_files:
+        for proc in procs:
+            try:
+                # WARNING: Process.open_files() has issues on Windows!
+                # https://psutil.readthedocs.io/en/latest/#psutil.Process.open_files
+                proc_files = tuple(path_fix(x.path) for x in proc.open_files())
+                if proc_files:
+                    for efile in check_files:
+                        if efile in proc_files:
+                            yield efile, proc.pid, proc.name()
+            except (AccessDenied, NoSuchProcess):  # pragma: no cover
+                pass
+
+
 def get_processes(pid, recursive=True):
-    """From a given PID create a psutil.Process object and lookup all of it's
+    """From a given PID create a psutil.Process object and lookup all of its
     children.
 
     Args:
@@ -404,15 +466,21 @@ def prepare_environment(target_dir, sanitizer_log, env_mod=None):
     # https://developer.mozilla.org/en-US/docs/Mozilla/Debugging/XPCOM_DEBUG_BREAK
     base["XPCOM_DEBUG_BREAK"] = "warn"
     base["XRE_NO_WINDOWS_CRASH_DIALOG"] = "1"
-    # apply envionment modifications
+    # apply environment modifications
     if env_mod is not None:
         assert isinstance(env_mod, dict)
         base.update(env_mod)
     # environment variables to skip if previously set in environ
     optional = (
-        "_RR_TRACE_DIR", "MOZ_CRASHREPORTER", "MOZ_CRASHREPORTER_NO_DELETE_DUMP",
-        "MOZ_CRASHREPORTER_NO_REPORT", "MOZ_CRASHREPORTER_SHUTDOWN",
-        "MOZ_SKIA_DISABLE_ASSERTS", "RUST_BACKTRACE", "XPCOM_DEBUG_BREAK")
+        "_RR_TRACE_DIR",
+        "MOZ_CRASHREPORTER",
+        "MOZ_CRASHREPORTER_NO_DELETE_DUMP",
+        "MOZ_CRASHREPORTER_NO_REPORT",
+        "MOZ_CRASHREPORTER_SHUTDOWN",
+        "MOZ_SKIA_DISABLE_ASSERTS",
+        "RUST_BACKTRACE",
+        "XPCOM_DEBUG_BREAK",
+    )
     # merge presets and modifications
     for env_name, env_value in base.items():
         if env_value is None:
@@ -448,11 +516,12 @@ def true_path(path):
     return normcase(realpath(path))
 
 
-def wait_on_files(wait_files, poll_rate=0.25, timeout=60):
+def wait_on_files(procs, wait_files, poll_rate=0.25, timeout=60):
     """Wait for files in wait_files to no longer be use by any process.
 
     Args:
-        path (iterable): Files that must no longer be open by a process.
+        procs (iterable(Process)): Processes to scan for open files.
+        wait_files (iterable(str)): Files that must no longer be open by a process.
         poll_rate (float): Amount of time in seconds to wait between checks.
         timeout (int): Amount of time in seconds to poll.
 
@@ -464,30 +533,31 @@ def wait_on_files(wait_files, poll_rate=0.25, timeout=60):
     wait_files = {true_path(x) for x in wait_files if isfile(x)}
     if not wait_files:
         return True
-    deadline = time() + timeout
-    # collect all blocking processes
-    procs = list()
-    for proc in process_iter(attrs=["pid", "open_files"]):
-        if not proc.info["open_files"]:
-            continue
-        # WARNING: Process.open_files() has issues on Windows!
-        # https://psutil.readthedocs.io/en/latest/#psutil.Process.open_files
-        if wait_files.intersection({true_path(x.path) for x in proc.info["open_files"]}):
-            try:
-                procs.append(Process(proc.info["pid"]))
-            except (AccessDenied, NoSuchProcess):  # pragma: no cover
-                pass
-    # only check previously blocking processes
     poll_rate = min(poll_rate, timeout)
-    while procs:
-        try:
-            if wait_files.intersection({true_path(x.path) for x in procs[-1].open_files()}):
-                if deadline <= time():
-                    LOG.debug("wait_on_files(timeout=%d) timed out", timeout)
-                    return False
-                sleep(poll_rate)
-                continue
-        except (AccessDenied, NoSuchProcess):  # pragma: no cover
-            pass
-        procs.pop()
-    return True
+    deadline = time() + timeout
+    while any(files_in_use(wait_files, true_path, procs)):
+        if deadline <= time():
+            LOG.debug("wait_on_files timeout (%ds)", timeout)
+            break
+        sleep(poll_rate)
+    else:
+        return True
+    for entry in files_in_use(wait_files, true_path, procs):
+        LOG.debug("%r open by %r (%d)", entry[0], entry[2], entry[1])
+    return False
+
+
+def warn_open(path):
+    """Output a message via `LOG.warning` for each file found to be open by a Process.
+    On Windows open files cannot be removed. Hopefully this can be used to help identify
+    the processes using the files and the underlying issue.
+
+    Args:
+        path (str): Path to scan for initial files.
+
+    Returns:
+        None
+    """
+    check = tuple(abspath(x.path) for x in scandir(path))
+    for entry in files_in_use(check, abspath, process_iter()):
+        LOG.warning("%s open by %r (%d)", entry[0], entry[2], entry[1])
